﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using Commons;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace Zerich.Micex
{
    class MicexConnectionsDeamon
    {
        List<Connection> connections;

        public void OpenConnections(IEnumerable<string> auth_strings)
        {
            List<int> connection_ids = new List<int>();
            foreach (String auth_string in auth_strings)
            {
                string user_id = auth_string.Split(new string[] { "UserId=" }, StringSplitOptions.RemoveEmptyEntries)[1].Split('\r')[0];

                // Устанавливаем связь с сервером
                StringBuilder err = new StringBuilder(); ;
                int value = MTEConnect(auth_string, err);
                if (value >= 0)
                {
                    connection_ids.Add(value);
                    Log.Push("Идентификатор {0} подключен", user_id);
                }
                else
                {
                    Log.Push(LogItemType.Error, "Идентификатор {0} не подключен", user_id);
                    Log.Push(LogItemType.Error, err.ToString());
                }
            }

            connections = new List<Connection>();
            if (connection_ids.IsNullOrEmpty())
                throw new MicexApiException("Ни одного соединения установить не удалось");
            else if (connection_ids.Count == 1)
            {
                connections.Add(new Connection { Id = connection_ids[0], Types = MicexConnectionType.All, IsBusy = false });
            }
            else
            {
                for (int i=0; i< connection_ids.Count - 1; i++)
                    connections.Add(new Connection { Id = connection_ids[i], Types = MicexConnectionType.PostOrders, IsBusy = false });
                connections.Add(new Connection { Id = connection_ids.Last(), Types = MicexConnectionType.RefreshTables, IsBusy = false });
            }
        }

        public void CloseConnections()
        {
            Stopwatch sw = Stopwatch.StartNew();
            while (connections.Count > 0)
            {
                int id = GetConnectionId(MicexConnectionType.All);
                if (id >= 0)
                {
                    MicexReturnCode code = (MicexReturnCode)MTEDisconnect(id);
                    if (code != MicexReturnCode.Ok)
                        Log.Push(LogItemType.Error, "Соединение закрыто с ошибкой, код {0}", code.ToString());
                    connections.RemoveAll(c => c.Id == id);
                }
            }
        }

        int tries_count = 1000;
        public int GetConnectionId(MicexConnectionType type)
        {            
                if (connections.IsNullOrEmpty())
                    return -1;

                for (int try_count = 0; try_count < tries_count; try_count++)
                {
                    lock (connections)
                    {
                        Connection con = connections.FirstOrDefault(c => (int)(c.Types & type) > 0 && !c.IsBusy);
                        if (con != null)
                        {
                            con.IsBusy = true;
                            return con.Id;
                        }
                    }

                    Thread.Sleep((int)Config.StandartWaitingTime.TotalMilliseconds / tries_count);
                }

            throw new MicexApiException("Дождаться свободного соединения со шлюзом не удалось");
        }

        public void ReleaseConnectionId(int id)
        {
            if (id < 0)
                return;

            lock (connections)
            {
                Connection connection = connections.SingleOrDefault(con => con.Id == id);
                if (connection != null)
                    connection.IsBusy = false;
            }
        }

        

        class Connection
        {
            public int Id;
            public MicexConnectionType Types;
            public bool IsBusy;
        }



        #region Служебное
        private static string ConvertAndDecode(byte[] bytes)
        {
            return Encoding.GetEncoding(1251).GetString(bytes).TrimEnd('\0');
        }

        [DllImport("mtesrl.dll", CharSet = CharSet.Ansi)]
        private static extern int MTEConnect(String pars, StringBuilder err);

        [DllImport("mtesrl.dll", CharSet = CharSet.Ansi)]
        private static extern int MTEDisconnect(int connectionId);
        #endregion
    }

    enum MicexConnectionType
    {
        RefreshTables = 1,
        PostOrders = 2,
        All = 3
    }
}
